@@ -59,11 +59,11 @@ static char *read_full_file(const char *filename) {
     return content;
 }
 
-inline bool is_leap_year(const int year) {
+static inline bool is_leap_year(const int year) {
     return (year % 4 == 0 && year % 100 != 0) || year % 400 == 0;
 }
 
-inline bool is_valid_date(const date_t *date) {
+static inline bool is_valid_date(const date_t *date) {
     unsigned int days_in_month[] = {31, 28, 31, 30, 31, 30,
                                     31, 31, 30, 31, 30, 31};
     if(is_leap_year(date->year)) {
@@ -73,6 +73,12 @@ inline bool is_valid_date(const date_t *date) {
            date->day >= 1 && date->day <= days_in_month[date->month-1];
 }
 
+static inline bool is_valid_codename(const char *codename) {
+    // Only codenames with lowercase ASCII letters are accepted
+    return strlen(codename) > 0 &&
+           strspn(codename, "abcdefghijklmnopqrstuvwxyz") == strlen(codename);
+}
+
 // Read an ISO 8601 formatted date
 static date_t *read_date(const char *s, int *failures, const char *filename,
                          const int lineno, const char *column) {
@@ -93,30 +99,32 @@ static date_t *read_date(const char *s, int *failures, const char *filename,
     return date;
 }
 
-inline bool date_ge(const date_t *date1, const date_t *date2) {
+static inline bool date_ge(const date_t *date1, const date_t *date2) {
     return date1->year > date2->year ||
            (date1->year == date2->year && date1->month > date2->month) ||
            (date1->year == date2->year && date1->month == date2->month &&
             date1->day >= date2->day);
 }
 
-inline bool created(const date_t *date, const distro_t *distro) {
+static inline bool created(const date_t *date, const distro_t *distro) {
     return distro->created && date_ge(date, distro->created);
 }
 
-inline bool released(const date_t *date, const distro_t *distro) {
+static inline bool released(const date_t *date, const distro_t *distro) {
     return *distro->version != '\0' &&
            distro->release && date_ge(date, distro->release);
 }
 
-inline bool eol(const date_t *date, const distro_t *distro) {
+static inline bool eol(const date_t *date, const distro_t *distro) {
     return distro->eol && date_ge(date, distro->eol) && (!distro->eol_server ||
            (distro->eol_server && date_ge(date, distro->eol_server)));
 }
 
 // Filter callbacks
 
-static bool filter_all() {
+static bool filter_all(const date_t *date, const distro_t *distro) {
+    (void)date;
+    (void)distro;
     return true;
 }
 
@@ -315,8 +323,12 @@ static void print_help(void) {
 \n\
 Options:\n\
   -h  --help         show this help message and exit\n\
-      --date=DATE    date for calculating the version (default: today)\n\
-  -a  --all          list all known versions\n\
+      --date=DATE    date for calculating the version (default: today)\n"
+#ifdef DEBIAN
+"      --alias=DIST   print the alias (stable, testing, unstable) relative to\n\
+                     the distribution codename passed as an argument\n"
+#endif
+"  -a  --all          list all known versions\n\
   -d  --devel        latest development version\n"
 #ifdef UBUNTU
 "      --lts          latest long term support (LTS) version\n"
@@ -331,14 +343,18 @@ Options:\n\
 #endif
 "      --unsupported  list of all unsupported stable versions\n\
   -c  --codename     print the codename (default)\n\
-  -r  --release      print the release version\n\
   -f  --fullname     print the full name\n\
+  -r  --release      print the release version\n\
 \n\
 See " NAME "(1) for more info.\n");
 }
 
-inline int not_exactly_one(void) {
-    fprintf(stderr, NAME ": You have to select exactly one of --all, --devel, "
+static inline int not_exactly_one(void) {
+    fprintf(stderr, NAME ": You have to select exactly one of "
+#ifdef DEBIAN
+            "--alias, "
+#endif
+            "--all, --devel, "
 #ifdef UBUNTU
             "--lts, "
 #endif
@@ -362,9 +378,13 @@ int main(int argc, char *argv[]) {
     int option;
     int option_index;
     int return_value = EXIT_SUCCESS;
+    int selected_filters = 0;
     bool (*filter_cb)(const date_t*, const distro_t*) = NULL;
     const distro_t *(*select_cb)(const distro_elem_t*) = NULL;
     void (*print_cb)(const distro_t*) = print_codename;
+#ifdef DEBIAN
+    char *alias_codename = NULL;
+#endif
 
     const struct option long_options[] = {
         {"help",        no_argument,       NULL, 'h' },
@@ -375,14 +395,15 @@ int main(int argc, char *argv[]) {
         {"supported",   no_argument,       NULL, 'S' },
         {"unsupported", no_argument,       NULL, 'U' },
         {"codename",    no_argument,       NULL, 'c' },
-        {"release",     no_argument,       NULL, 'r' },
         {"fullname",    no_argument,       NULL, 'f' },
-#ifdef UBUNTU
-        {"lts",         no_argument,       NULL, 'L' },
-#endif
+        {"release",     no_argument,       NULL, 'r' },
 #ifdef DEBIAN
+        {"alias",       required_argument, NULL, 'A' },
         {"oldstable",   no_argument,       NULL, 'o' },
         {"testing",     no_argument,       NULL, 't' },
+#endif
+#ifdef UBUNTU
+        {"lts",         no_argument,       NULL, 'L' },
 #endif
         {NULL,          0,                 NULL, '\0' }
     };
@@ -400,8 +421,27 @@ int main(int argc, char *argv[]) {
     while ((option = getopt_long(argc, argv, short_options,
                                  long_options, &option_index)) != -1) {
         switch (option) {
+#ifdef DEBIAN
+            case 'A':
+                // Only long option --alias is used
+                if(unlikely(alias_codename != NULL)) {
+                    fprintf(stderr, NAME ": --alias requested multiple times.\n");
+                    free(date);
+                    return EXIT_FAILURE;
+                }
+                if(!is_valid_codename(optarg)) {
+                    fprintf(stderr, NAME ": invalid distribution codename `%s'\n",
+                            optarg);
+                    free(date);
+                    return EXIT_FAILURE;
+                }
+                selected_filters++;
+                alias_codename = optarg;
+                break;
+#endif
+
             case 'a':
-                if(unlikely(filter_cb != NULL)) return not_exactly_one();
+                selected_filters++;
                 filter_cb = filter_all;
                 select_cb = NULL;
                 break;
@@ -411,7 +451,7 @@ int main(int argc, char *argv[]) {
                 break;
 
             case 'd':
-                if(unlikely(filter_cb != NULL)) return not_exactly_one();
+                selected_filters++;
                 filter_cb = filter_devel;
 #ifdef UBUNTU
                 select_cb = select_latest_created;
@@ -444,12 +484,13 @@ int main(int argc, char *argv[]) {
 
             case 'h':
                 print_help();
+                free(date);
                 return EXIT_SUCCESS;
 
 #ifdef UBUNTU
             case 'L':
                 // Only long option --lts is used
-                if(unlikely(filter_cb != NULL)) return not_exactly_one();
+                selected_filters++;
                 filter_cb = filter_lts;
                 select_cb = select_latest_release;
                 break;
@@ -457,7 +498,7 @@ int main(int argc, char *argv[]) {
 
 #ifdef DEBIAN
             case 'o':
-                if(unlikely(filter_cb != NULL)) return not_exactly_one();
+                selected_filters++;
                 filter_cb = filter_oldstable;
                 select_cb = select_oldstable;
                 break;
@@ -468,21 +509,21 @@ int main(int argc, char *argv[]) {
                 break;
 
             case 's':
-                if(unlikely(filter_cb != NULL)) return not_exactly_one();
+                selected_filters++;
                 filter_cb = filter_stable;
                 select_cb = select_latest_release;
                 break;
 
             case 'S':
                 // Only long option --supported is used
-                if(unlikely(filter_cb != NULL)) return not_exactly_one();
+                selected_filters++;
                 filter_cb = filter_supported;
                 select_cb = NULL;
                 break;
 
 #ifdef DEBIAN
             case 't':
-                if(unlikely(filter_cb != NULL)) return not_exactly_one();
+                selected_filters++;
                 filter_cb = filter_testing;
                 select_cb = select_latest_created;
                 break;
@@ -490,7 +531,7 @@ int main(int argc, char *argv[]) {
 
             case 'U':
                 // Only long option --unsupported is used
-                if(unlikely(filter_cb != NULL)) return not_exactly_one();
+                selected_filters++;
                 filter_cb = filter_unsupported;
                 select_cb = NULL;
                 break;
@@ -500,6 +541,11 @@ int main(int argc, char *argv[]) {
                     // Long option failed
                     fprintf(stderr, NAME ": unrecognized option `%s'\n",
                             argv[optind-1]);
+#ifdef DEBIAN
+                } else if(optopt == 'A') {
+                    fprintf(stderr, NAME ": option `--alias' requires "
+                            "an argument DIST\n");
+#endif
                 } else if(optopt == 'D') {
                     fprintf(stderr, NAME ": option `--date' requires "
                             "an argument DATE\n");
@@ -507,12 +553,14 @@ int main(int argc, char *argv[]) {
                     fprintf(stderr, NAME ": unrecognized option `-%c'\n",
                             optopt);
                 }
+                free(date);
                 return EXIT_FAILURE;
                 break;
 
             default:
                 fprintf(stderr, NAME ": getopt returned character code %i. "
                         "Please file a bug report.\n", option);
+                free(date);
                 return EXIT_FAILURE;
         }
     }
@@ -523,10 +571,12 @@ int main(int argc, char *argv[]) {
             fprintf(stderr, " %s", argv[i]);
         }
         fprintf(stderr, "\n");
+        free(date);
         return EXIT_FAILURE;
     }
 
-    if(unlikely(filter_cb == NULL)) {
+    if(unlikely(selected_filters != 1)) {
+        free(date);
         return not_exactly_one();
     }
 
@@ -541,20 +591,48 @@ int main(int argc, char *argv[]) {
 
     distro_list = read_data(DATA_DIR "/" CSV_NAME ".csv", &content);
     if(unlikely(distro_list == NULL)) {
+        free(date);
         return EXIT_FAILURE;
     }
 
+#ifdef DEBIAN
+    if(alias_codename) {
+        const distro_t *stable = get_distro(distro_list, date, filter_stable,
+                                            select_latest_release);
+        const distro_t *testing = get_distro(distro_list, date, filter_testing,
+                                             select_latest_created);
+        const distro_t *unstable = get_distro(distro_list, date, filter_devel,
+                                              select_first);
+        if(unlikely(stable == NULL || testing == NULL || unstable == NULL)) {
+            fprintf(stderr, NAME ": " OUTDATED_ERROR "\n");
+            return_value = EXIT_FAILURE;
+        } else if(strcmp(stable->series, alias_codename) == 0) {
+            printf("stable\n");
+        } else if(strcmp(testing->series, alias_codename) == 0) {
+            printf("testing\n");
+        } else if(strcmp(unstable->series, alias_codename) == 0) {
+            printf("unstable\n");
+        } else {
+            printf("%s\n", alias_codename);
+        }
+        free(date);
+        free_data(distro_list, &content);
+        return return_value;
+    }
+#endif
+
     if(select_cb == NULL) {
         filter_data(distro_list, date, filter_cb, print_cb);
     } else {
         selected = get_distro(distro_list, date, filter_cb, select_cb);
         if(selected == NULL) {
-            fprintf(stderr, NAME ": Distribution data outdated.\n");
+            fprintf(stderr, NAME ": " OUTDATED_ERROR "\n");
             return_value = EXIT_FAILURE;
         } else {
             print_cb(selected);
         }
     }
+    free(date);
     free_data(distro_list, &content);
     return return_value;
 }
